package safehome.device.netcamera;

import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import javax.imageio.ImageIO;

public class CameraPacket implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 8040572156190517223L;
	//private int size = 0;
	private int type = 0;
	private byte[] serializedImage = null;
	
	public static final int CONTROL_SEND_IMAGE = 10;
	public static final int CONTROL_PAN_LEFT = 1;
	public static final int CONTROL_PAN_RIGHT = 2;
	public static final int CONTROL_ZOOM_IN = 3;
	public static final int CONTROL_ZOOM_OUT = 4;
	
	
	public CameraPacket(int type){
		this.type = type;
		this.serializedImage = null;
	}
	public CameraPacket(int type, byte[] serializedImage) {
		this.type = type;
		this.serializedImage = serializedImage;
	}
	public CameraPacket(int type, BufferedImage img){
		this.type = type;
		this.serializedImage = toBytes(img);
	}
	
	public int getType() {
		return type;
	}

	public BufferedImage getBufferedImage(){
		return toBufferedImage(serializedImage);
	}

	public byte[] toBytes(){
		return toBytes(this);
	}
	public static CameraPacket toPacket(byte[] data){
		ByteArrayInputStream bais = new ByteArrayInputStream(data);
		CameraPacket packet = null;
		try{
			ObjectInputStream ois = new ObjectInputStream(bais);
			packet = (CameraPacket) ois.readObject();
		}catch(IOException ioe){
			ioe.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		return packet;
	}
	
	private byte[] toBytes(Object o){  
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		byte[] ret = null;
		try{  
			ObjectOutputStream oos = new ObjectOutputStream(baos);  
			oos.writeObject(o);  
			oos.flush();
			baos.flush();
			ret = baos.toByteArray();
		}catch(IOException ioe){  
			ioe.printStackTrace();  
		}  
		return ret;
	}  
	private byte[] toBytes(BufferedImage img){
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		byte[] ret = null;
		try{
			ImageIO.write(img, "jpeg", baos);
			baos.flush();
			ret = baos.toByteArray();
		}catch(IOException ioe){
			ioe.printStackTrace();
		}catch(IllegalArgumentException iae){
			/* empty */
		}
		return ret;
		
	}
	
	private BufferedImage toBufferedImage(byte[] data){
		ByteArrayInputStream bios = new ByteArrayInputStream(data);
		BufferedImage img = null;
		
		try {
			img = ImageIO.read(bios);
		} catch (IOException e) {
			e.printStackTrace();
		}
		

		return img;
		
	}

}
