package safehome.netcameraviewer;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferedImage;

import javax.swing.JFrame;

import safehome.device.netcamera.NetworkCamera;

 
/**
 * <code>CameraViewComponent</code> class is a simple java component 
 * which has <code>{@link BufferedImage}</code> from <code>NetworkCamera</code>. 
 * This java component can be attached to any <code>{@link JFrame}</code>. 
 * @author cs350TA
 *
 */
class CameraViewComponent extends Component implements Runnable
{
	// Network camera instance
	private NetworkCamera camera = null;
	
	
	/**
	 * Constructs camera view component. This component gets camera images 
	 * from the network camera and shows those images. 
	 * @param camera the network camera instance to be shown
	 */
	public CameraViewComponent(NetworkCamera camera){
		this.camera = camera;
		
		new Thread(this).start();
	}
	
	
    /** 
     * Paints this component. This method gets new image from camera device and re-draw image to the component.
     * @param g the graphics context to use for painting
     * @see java.awt.Component#paint(java.awt.Graphics)
     */
    public void paint(Graphics g) 
    {
        // Draw the image stretched to exactly cover the size of the drawing area.
        Dimension size = getSize();
        BufferedImage img = camera.getView();
        
        g.drawImage(img, 
                    0, 0, size.width, size.height,
                    0, 0, img.getWidth(null), img.getHeight(null),
                    null);                    
    }
     

	/** 
	 * When an object implementing interface <code>Runnable</code> is used to create a thread, 
	 * starting the thread causes the object's run method to be called in that separately executing thread. 
	 * 
	 * This thread updates view of the camera in periodic interval 100ms.
	 * @see java.lang.Runnable#run()
	 */
    
	public void run()
	{
		while(true)
		{
			try
			{
				Thread.sleep(100);
			}
			catch(InterruptedException e)
			{
			}
			repaint();
		}
	}
    
    /*
	 * meaningless member variable in this application
	 * prevent java compiler warning message
	 */
	static final long serialVersionUID = 45634;
}